{*******************************************************}
{                                                       }
{       Delphi Runtime Library                          }
{       Windows NT Process Enumeration                  }
{                                                       }
{       Copyright (C) 2000 MichaL MutL                  }
{                                                       }
{*******************************************************}

unit EnumsNT;

interface

uses Windows, Classes, SysUtils, ShellAPI, PerfLibNT;

type
  PNTProcess = ^TNTProcess;
  TNTProcess = record
    PID :longword;
    Name,
    ImageName,
    Domain,
    User :string;
    CPUTime,
    KernelTime,
    UserTime,
    ElapsedTime :int64;
    PriorityBase :integer;
    HandleCount,
    HeapID,
    ThreadCount :longword;
    PageFaultCount,
    PrivateSpace,
    PeakWorkingSetSize,
    WorkingSetSize,
    PagedPoolSize,
    NonPagedPoolSize,
    VirtualAddressSpaceSize,
    PeakVirtualAddressSpaceSize,
    PageFileUsage,
    PeakPageFileUsage,
    ParentPID,
    IOReadOps,
    IOWriteOps,
    IODataOps,
    IOOtherOps,
    IOReadB,
    IOWriteB,
    IODataB,
    IOOtherB :integer;
    Modules,
    Threads,
    HeapLists :tstringlist;
  end;

  PNTModule = ^TNTModule;
  TNTModule = record
    Name,
    ImageName: string;
    ID,
    EntryPoint,
    LoadAddress,
    GlobalUsage,
    ProcessUsage,
    BaseSize,
    Handle,
    ImageSize :longword;
  end;

  PNTThread = ^TNTThread;
  TNTThread = record
    ID,
    Status,
    WaitReason,
    PriorityBase,
    PriorityDelta,
    StartAddr,
    CntxtSwtcs :longword;
    CPUTime :int64;
    szStatus,
    szWaitReason :string;
  end;

  procedure GetNTProcessList(APL: TPerfLibNT; var NTProcesslist :tstringlist);
  procedure GetNTProcessInfo(var Process: TNTProcess; APL: TPerfLibNT; APID :integer; ExtInfo :Boolean);

const
  SProcsPO = 'Process';
    SProcsIDPC = 'ID Process';
    SProcsPTPC = '% Processor Time';
    SProcsKTPC = '% Privileged Time';
    SProcsUTPC = '% User Time';
    SProcsWSPC = 'Working Set';
    SProcsWSPPC = 'Working Set Peak';
    SProcsPRPC = 'Priority Base';
    SProcsTCPC = 'Thread Count';
    SProcsPFPC = 'Page Faults/sec';
    SProcsPBPC = 'Private Bytes';
    SProcsPBPPC = 'Private Bytes Peak';
    SProcsPFBPC = 'Page File Bytes';
    SProcsPFBPPC = 'Page File Bytes Peak';
    SProcsETPC = 'Elapsed Time';
    SProcsPPBPC = 'Pool Paged Bytes';
    SProcsPNBPC = 'Pool NonPaged Bytes';
    SProcsHCPC = 'Handle Count';
    SProcsVBPC = 'Virtual Bytes';
    SProcsVBPPC = 'Virtual Bytes Peak';
//Win 2K
    SProcsCIDPC = 'Creating Process ID';
    SProcsIOROPC = 'IO Read Operations/sec';
    SProcsIOWOPC = 'IO Write Operations/sec';
    SProcsIODOPC = 'IO Data Operations/sec';
    SProcsIOOOPC = 'IO Other Operations/sec';
    SProcsIORBPC = 'IO Read Bytes/sec';
    SProcsIOWBPC = 'IO Write Bytes/sec';
    SProcsIODBPC = 'IO Data Bytes/sec';
    SProcsIOOBPC = 'IO Other Bytes/sec';

  SThrdsPO = 'Thread';
    SThrdsPIDPC = 'ID Process';
    SThrdsIDPC = 'ID Thread';
    SThrdsPTPC = '% Processor Time';
    SThrdsKTPC = '% Privileged Time';
    SThrdsUTPC = '% User Time';
    SThrdsCSPC = 'Context Switches/sec';
    SThrdsETPC = 'Elapsed Time';
    SThrdsPCPC = 'Priority Current';
    SThrdsPBPC = 'Priority Base';
    SThrdsSAPC = 'Start Address';
    SThrdsTSPC = 'Thread State';
    SThrdsTWPC = 'Thread Wait Reason';

implementation

uses PSAPI, ToolHelp32, Registry, Enums9x;

procedure GetNTProcessList;
var
  pc: TPerfCounter;
  po: TPerfObject;
  i :integer;
  ntp :pntprocess;
begin
  while ntprocesslist.count>0 do begin
    ntp:=pntprocess(ntprocesslist.objects[ntprocesslist.count-1]);
    if assigned(ntp^.Modules) then begin
      while ntp^.Modules.count>0 do begin
        dispose(pntmodule(ntp^.Modules.objects[ntp^.Modules.count-1]));
        ntp^.Modules.delete(ntp^.Modules.count-1);
      end;
      ntp^.Modules.Free;
    end;
    if assigned(ntp^.Threads) then begin
      while ntp^.Threads.count>0 do begin
        dispose(pntthread(ntp^.Threads.objects[ntp^.Threads.count-1]));
        ntp^.Threads.delete(ntp^.Threads.count-1);
      end;
      ntp^.Threads.Free;
    end;
    if assigned(ntp^.Heaplists) then begin
      while ntp^.Heaplists.count>0 do begin
        dispose(pheaplist(ntp^.heaplists.objects[ntp^.heaplists.count-1]));
        ntp^.heaplists.delete(ntp^.heaplists.count-1);
      end;
      ntp^.Heaplists.Free;
    end;
    dispose(ntp);
    ntprocesslist.delete(ntprocesslist.count-1);
  end;
  with APL do begin
    TakeSnapshot;
    po:=Objects[GetIndexByName(SProcsPO)];
    for i:=0 to po.InstanceCount-2 do begin
      new(ntp);
      ntp^.modules:=tstringlist.Create;
      ntp^.HeapLists:=TStringList.Create;
      ntp^.Threads:=tstringlist.Create;
      with ntp^ do begin
        pc:=po.Counters[po.GetCntrIndexByName(SProcsIDPC)];
        PID:=PInteger(pc.Data[i])^;
      end;
      getntprocessinfo(ntp^,APL,ntp^.PID,false);
      ntprocesslist.addobject(inttostr(ntp^.PID),@ntp^);
    end;
  end;
end;

procedure GetNTProcessInfo;
var
  mi :tmoduleinfo;
  pc: TPerfCounter;
  po: TPerfObject;
  ds,PID,ntpi,i,j,k,p :integer;
  ntm :pntmodule;
  ntt :pntthread;
  ph :thandle;
  sz :pchar;
  n :longword;
  p95: T95Process;
begin
  with APL do begin
    n:=255;
    sz:=stralloc(n);
    if ExtInfo then
      TakeSnapshot;
    ntpi:=-1;
    po:=Objects[GetIndexByName(SProcsPO)];
    pc:=nil;
    for i:=0 to po.InstanceCount-2 do begin
      pc:=po.Counters[po.GetCntrIndexByName(SProcsIDPC)];
      PID:=PInteger(pc.Data[i])^;
      if APID=PID then begin
        ntpi:=i;
        break;
      end;
    end;
    if Assigned(pc) and (ntpi>-1) then
      with Process do begin
        if not Assigned(Modules) then
          Modules:=tstringlist.Create;
        if not Assigned(HeapLists) then
          HeapLists:=TStringList.Create;
        if not Assigned(Threads) then
          Threads:=tstringlist.Create;
        PID:=PInteger(pc.Data[ntpi])^;
        Name:=po.Instances[ntpi].Name;
        if pid>10 then
          Name:=Name+'.exe';
        pc:=po.Counters[po.GetCntrIndexByName(SProcsPTPC)];
        CPUTime:=PLARGE_INTEGER(pc.Data[ntpi])^.QuadPart div Timer100n;
        pc:=po.Counters[po.GetCntrIndexByName(SProcsWSPC)];
        WorkingSetSize:=PInteger(pc.Data[ntpi])^;
        pc:=po.Counters[po.GetCntrIndexByName(SProcsPRPC)];
        PriorityBase:=PInteger(pc.Data[ntpi])^;
        pc:=po.Counters[po.GetCntrIndexByName(SProcsTCPC)];
        ThreadCount:=PInteger(pc.Data[ntpi])^;
        if extinfo then begin
          if Is2000 then
            p95:=Get95ProcessInfo(PID,extinfo);
          ph:=OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ,false,PID);
          getuseranddomainname(ph,User,Domain);
          if (ph<>0) and (pid>10) and PSAPILoaded then
            GetModuleFilenameEx(ph,0,sz,n)
          else
            strpcopy(sz,ExpandFilename(filesearch(Name,GetWinSysDir)));
          ImageName:=strpas(sz);
          if not PSAPILoaded and is2000 then
            ImageName:=p95.ImageName;
          if not FileExists(ImageName) then begin
            p:=Pos('\??\',ImageName);
            if p>0 then
              Delete(ImageName,1,4)
            else
              ImageName:=ExpandEnvVars(ImageName);
          end;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsKTPC)];
          KernelTime:=PLARGE_INTEGER(pc.Data[ntpi])^.QuadPart div timer100n;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsUTPC)];
          UserTime:=PLARGE_INTEGER(pc.Data[ntpi])^.QuadPart div timer100n;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsPFPC)];
          PageFaultCount:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsPBPC)];
          PrivateSpace:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsWSPPC)];
          PeakWorkingSetSize:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsPPBPC)];
          PagedPoolSize:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsPNBPC)];
          NonPagedPoolSize:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsVBPC)];
          VirtualAddressSpaceSize:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsVBPPC)];
          PeakVirtualAddressSpaceSize:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsPFBPC)];
          PageFileUsage:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsPFBPPC)];
          PeakPageFileUsage:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsHCPC)];
          HandleCount:=PInteger(pc.Data[ntpi])^;
          pc:=po.Counters[po.GetCntrIndexByName(SProcsETPC)];
          ElapsedTime:=abs(APL.PerfTime.QuadPart-PLARGE_INTEGER(pc.Data[ntpi])^.quadpart) div APL.PerfFreq.QuadPart;
          if Is2000 then begin
            pc:=po.Counters[po.GetCntrIndexByName(SProcsCIDPC)];
            ParentPID:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIOROPC)];
            IOReadOps:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIOWOPC)];
            IOWriteOps:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIODOPC)];
            IODataOps:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIOOOPC)];
            IOOtherOps:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIORBPC)];
            IOReadB:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIOWBPC)];
            IOWriteB:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIODBPC)];
            IODataB:=PInteger(pc.Data[ntpi])^;
            pc:=po.Counters[po.GetCntrIndexByName(SProcsIOOBPC)];
            IOOtherB:=PInteger(pc.Data[ntpi])^;
          end;
          if PSAPILoaded then begin
            ReallocMem(ModuleList,0);
            if ph<>0 then begin
              ReallocMem (ModuleList,65536);
              if not EnumProcessModules(ph,ModuleList,65536,ds) then
                ds:=0;
              ReallocMem(ModuleList,ds);
            end else
              ReallocMem(ModuleList,0);
            if (ph<>0) then begin
              for j:=0 to ds div sizeof(hInst)-1 do begin
                new(ntm);
                with ntm^ do begin
                  k:=PHInst(pchar(ModuleList)+j*sizeof(HInst))^;
                  GetModuleInformation(ph,k,mi,sizeof(mi));
                  GetModuleFileNameEx(ph,k,sz,n);
                  ImageName:=strpas(sz);
                  if not FileExists(ImageName) then begin
                    p:=Pos('\??\',ImageName);
                    if p>0 then
                      Delete(ImageName,1,4)
                    else
                      ImageName:=ExpandEnvVars(ImageName);
                  end;
                  Name:=ExtractFilename(ImageName);
                  LoadAddress:=longword(mi.lpBaseOfDll);
                  ImageSize:=mi.SizeOfImage;
                  EntryPoint:=longword(mi.entrypoint);
                  modules.addobject(name,@ntm^);
                end;
              end;
            end;
          end;

          if is2000 then begin
            for i:=0 to p95.Modules.Count-1 do begin
              k:=Modules.IndexOf(p95.Modules[i]);
              if k>-1 then
                ntm:=PNTModule(Modules.Objects[k])
              else
                new(ntm);
              with ntm^ do begin
                ID:=P95Module(p95.Modules.Objects[i])^.ID;
                Name:=P95Module(p95.Modules.Objects[i])^.Name;
                ImageName:=P95Module(p95.Modules.Objects[i])^.ImageName;
                if not FileExists(ImageName) then begin
                  p:=Pos('\??\',ImageName);
                  if p>0 then
                    Delete(ImageName,1,4)
                  else
                    ImageName:=ExpandEnvVars(ImageName);
                end;
                GlobalUsage:=P95Module(p95.Modules.Objects[i])^.GlobalUsage;
                ProcessUsage:=P95Module(p95.Modules.Objects[i])^.ProcessUsage;
                BaseSize:=P95Module(p95.Modules.Objects[i])^.BaseSize;
                Handle:=P95Module(p95.Modules.Objects[i])^.Handle;
                EntryPoint:=P95Module(p95.Modules.Objects[i])^.EntryPoint;
                if k=-1 then
                  modules.addobject(Name,@ntm^);
              end;
            end;
            HeapID:=p95.HeapID;
            HeapLists:=p95.HeapLists;
          end;
          po:=Objects[GetIndexByName(SThrdsPO)];
          for j:=0 to po.InstanceCount-2 do begin
            pc:=po.Counters[po.GetCntrIndexByName(SThrdsPIDPC)];
            k:=PInteger(pc.Data[j])^;
            if k=Process.pid then begin
              new(ntt);
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsIDPC)];
              ntt^.ID:=PInteger(pc.Data[j])^;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsTSPC)];
              ntt^.Status:=PInteger(pc.Data[j])^;
              case ntt^.Status of
                0: ntt^.szStatus:='Init';
                1: ntt^.szStatus:='Ready';
                2: ntt^.szStatus:='Running';
                3: ntt^.szStatus:='StandBy';
                4: ntt^.szStatus:='Terminate';
                5: ntt^.szStatus:='Wait';
                6: ntt^.szStatus:='Transition';
              else
                ntt^.szStatus:='Unknown';
              end;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsTWPC)];
              ntt^.WaitReason:=PInteger(pc.Data[j])^;
              case ntt^.WaitReason of
                0,7: ntt^.szWaitReason:='Executive';
                1,8: ntt^.szWaitReason:='Free Page';
                2,9: ntt^.szWaitReason:='Page In';
                3,10: ntt^.szWaitReason:='Pool Alloc';
                4,11: ntt^.szWaitReason:='Exec Delay';
                5,12: ntt^.szWaitReason:='Suspend';
                6,13: ntt^.szWaitReason:='User Request';
                14: ntt^.szWaitReason:='Event Pair High';
                15: ntt^.szWaitReason:='Event Pair Low';
                16: ntt^.szWaitReason:='LPC Receive';
                17: ntt^.szWaitReason:='LPT Reply';
                18: ntt^.szWaitReason:='Virtual Memory';
                19: ntt^.szWaitReason:='Page Out';
              else
                ntt^.szWaitReason:='Unknown';
              end;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsPBPC)];
              ntt^.PriorityBase:=PInteger(pc.Data[j])^;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsPCPC)];
              ntt^.PriorityDelta:=PInteger(pc.Data[j])^;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsSAPC)];
              ntt^.StartAddr:=PInteger(pc.Data[j])^;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsCSPC)];
              ntt^.CntxtSwtcs:=PInteger(pc.Data[j])^;
              pc:=po.Counters[po.GetCntrIndexByName(SThrdsPTPC)];
              ntt^.CPUTime:=PLARGE_INTEGER(pc.Data[j])^.QuadPart div timer100n;
              Process.threads.addobject(inttostr(ntt^.ID),@ntt^);
            end else
              if Process.threads.count>0 then
                break;
          end;
        end;
      end;
    strdispose(sz);
  end;
end;

end.
