unit ProcView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, Buttons, ExtCtrls, StdCtrls, Menus, PerfLibNT;

type
  TfrmProcView = class(TForm)
    pClient1: TPanel;
    lvpv: TListView;
    MainMenu1: TMainMenu;
    sb: TStatusBar;
    Dock971: TPanel;
    sbRefresh: TSpeedButton;
    sbKill: TSpeedButton;
    sbClose: TSpeedButton;
    File1: TMenuItem;
    mmClose: TMenuItem;
    Process1: TMenuItem;
    mmKill: TMenuItem;
    mmRefresh: TMenuItem;
    N2: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure cmClose(Sender: TObject);
    procedure cmRefresh(Sender: TObject);
    procedure cmTerminate(Sender: TObject);
    procedure lvpvKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    ProcessList :TStringList;
    FPerfLib: TPerfLibNT;
  public
    { Public declarations }
    procedure InitProcessList;
    procedure Refresh;
  end;

var
  frmProcView: TfrmProcView;

implementation

uses Enums9x, EnumsNT;

{$R *.DFM}

resourcestring
  SNAME = 'Name';
  SCPU = 'CPU';
  STHRD = 'Threads';
  SPRIOR = 'Priority';
  SPID = 'PID';
  SMEMORY = 'Memory';
  SREFS = 'Usage';

  SWARN = 'WARNING: Terminating a process can cause undesired '#13#10+
                    'results including loss of data and system instability. The '#13#10+
                    'process will not be given the chance to save its state or '#13#10+
                    'data before it is terminated. Are you sure you want to '#13#10+
                    'terminate the process?';
  SCannotTerm = 'Cannot terminate this process.';

function IsNT :boolean;
var
  OS :TOSVersionInfo;
begin
  ZeroMemory(@OS,SizeOf(OS));
  OS.dwOSVersionInfoSize:=SizeOf(OS);
  GetVersionEx(OS);
  result:=OS.dwPlatformId=VER_PLATFORM_WIN32_NT;
end;

function FormatSeconds(TotalSeconds :comp; WholeSecondsOnly, DisplayAll, DTFormat :Boolean) :String;
var
 lcenturies,lyears,lmonths,lminutes,lhours,ldays,lweeks :word;
 lSecs :double;
 s :array[1..8] of string;
 SecondsPerCentury :comp;
 FS :string;
begin
  if WholeSecondsOnly then
    FS:='%.0f'
  else
    FS:='%.2f';
  SecondsPerCentury:=36550 * 24;
  SecondsPerCentury:= SecondsPerCentury * 3600;
  lcenturies:=Trunc(TotalSeconds / SecondsPerCentury);
  TotalSeconds:=TotalSeconds-(lcenturies * SecondsPerCentury);
  lyears:=Trunc(TotalSeconds / (365.5 * 24 * 3600));
  TotalSeconds:=TotalSeconds-(lyears * (365.5 * 24 * 3600));
  lmonths:=Trunc(TotalSeconds / (31 * 24 * 3600));
  TotalSeconds:=TotalSeconds-(lmonths * (31 * 24 * 3600));
  lweeks:=Trunc(TotalSeconds / (7 * 24 * 3600));
  TotalSeconds:=TotalSeconds-(lweeks * (7 * 24 * 3600));
  ldays:=Trunc(TotalSeconds / (24 * 3600));
  TotalSeconds:=TotalSeconds-(ldays * (24 * 3600));
  lhours:=Trunc(TotalSeconds / 3600);
  TotalSeconds:=TotalSeconds-(lhours * 3600);
  lminutes:=Trunc(TotalSeconds / 60);
  TotalSeconds:=TotalSeconds-(lminutes * 60);
  If WholeSecondsOnly then
    lsecs:=Trunc(TotalSeconds)
  else
    lsecs:=TotalSeconds;
  if lCenturies=1 then
    s[1]:=' Century, '
  else
    s[1]:=' Centuries, ';
  if lyears=1 then
    s[2]:=' Year, '
  else
    s[2]:=' Years, ';
  if lmonths=1 then
    s[3]:=' Month, '
  else
    s[3]:=' Months, ';
  if lweeks=1 then
    s[4]:=' Week, '
  else
    s[4]:=' Weeks, ';
  if ldays=1 then
    s[5]:=' Day, '
  else
    s[5]:=' Days, ';
  if lhours=1 then
    s[6]:=' Hour, '
  else
    s[6]:=' Hours, ';
  if lminutes=1 then
    s[7]:=' Minute, '
  else
    s[7]:=' Minutes, ';
  if lsecs=1 then
    s[8]:=' Second.'
  else
    s[8]:=' Seconds.';
  If DisplayAll then begin
    if dtformat then
      result:=Format('%2.2d.%2.2d.%2.2d %2.2d:%2.2d:%2.2d',
                     [lyears,lmonths,ldays+lweeks*7,lhours,lminutes,round(lSecs)])
    else
      Result:= Format('%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s' + FS + '%s',
                     [lcenturies,s[1],lyears,s[2],lmonths,s[3],lweeks,s[4],ldays,s[5],lhours,s[6],lminutes,s[7],lSecs,s[8]]);

  end else begin
    if dtformat then
      result:=Format('%2.2d:%2.2d:%2.2d',
                     [lhours,lminutes,round(lSecs)])
    else begin
      if lCenturies>=1 then
        Result:= Format('%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s' + FS + '%s',
                        [lcenturies,s[1],lyears,s[2],lmonths,s[3],lweeks,s[4],ldays,s[5],lhours,s[6],lminutes,s[7],lsecs,s[8]])
      else
        if lyears>=1 then
          Result:= Format('%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s' + FS + '%s',
                          [lyears,s[2],lmonths,s[3],lweeks,s[4],ldays,s[5],lhours,s[6],lminutes,s[7],lsecs,s[8]])
      else
       if lmonths>=1 then
         Result:= Format('%.0d%s%.0d%s%.0d%s%.0d%s%.0d%s' + FS + '%s',
                         [lmonths,s[3],lweeks,s[4],ldays,s[5],lhours,s[6],lminutes,s[7],lsecs,s[8]])
       else
         if lweeks>=1 then
           Result:= Format('%.0d%s%.0d%s%.0d%s%.0d%s' + FS + '%s',
                           [lweeks,s[4],ldays,s[5],lhours,s[6],lminutes,s[7],lsecs,s[8]])
         else
           if ldays>=1 then
             Result:= Format('%.0d%s%.0d%s%.0d%s' + FS + '%s',
                             [ldays,s[5],lhours,s[6],lminutes,s[7],lsecs,s[8]])
           else
             if lhours>=1 then
               Result:= Format('%.0d%s%.0d%s' + FS + '%s',
                               [lhours,s[6],lminutes,s[7],lsecs,s[8]])
             else
               if lminutes>=1 then
                 Result:= Format('%.0d%s' + FS + '%s',[lminutes,s[7],lsecs,s[8]])
               else
                 Result:= Format(FS + '%s',[lsecs,s[8]]);
    end;
  end;
end;

procedure TfrmProcView.FormCreate(Sender: TObject);
begin
  if IsNT then
    FPerfLib:=TPerfLibNT.Create;
  processlist:=tstringlist.create;
  initprocesslist;
  cmrefresh(nil);
end;

procedure TfrmProcView.cmClose(Sender: TObject);
begin
  close;
end;

procedure TfrmProcView.InitProcessList;
begin
  screen.cursor:=crhourglass;
  lvpv.items.clear;
  lvpv.columns.clear;
  if isnt then begin
    with lvpv.columns.add do begin
      caption:=SNAME;
      alignment:=taleftjustify;
      width:=100;
    end;
    with lvpv.columns.add do begin
      caption:=SPID;
      alignment:=tarightjustify;
      width:=40;
    end;
    with lvpv.columns.add do begin
      caption:=SCPU;
      alignment:=tarightjustify;
      width:=70;
    end;
    with lvpv.columns.add do begin
      caption:=SMEMORY;
      alignment:=tarightjustify;
      width:=70;
    end;
    with lvpv.columns.add do begin
      caption:=SPRIOR;
      alignment:=tarightjustify;
      width:=50;
    end;
    with lvpv.columns.add do begin
      caption:=STHRD;
      alignment:=tarightjustify;
      width:=60;
    end;
  end else begin
    with lvpv.columns.add do begin
      caption:=SNAME;
      alignment:=taleftjustify;
      width:=120;
    end;
    with lvpv.columns.add do begin
      caption:=SPID;
      alignment:=tarightjustify;
      width:=80;
    end;
    with lvpv.columns.add do begin
      caption:=SPRIOR;
      alignment:=tarightjustify;
      width:=50;
    end;
    with lvpv.columns.add do begin
      caption:=SREFS;
      alignment:=tarightjustify;
      width:=50;
    end;
    with lvpv.columns.add do begin
      caption:=STHRD;
      alignment:=tarightjustify;
      width:=60;
    end;
  end;
  screen.cursor:=crdefault;
end;

procedure TfrmProcView.cmRefresh(Sender: TObject);
begin
  Refresh;
end;

procedure TfrmProcView.cmTerminate(Sender: TObject);
var
  ph :thandle;
  pid :integer;
begin
  if assigned(lvpv.selected) then begin
    if isNT  then begin
      pid:=strtoint(lvpv.selected.subitems[0])
    end else
      pid:=strtoint('$'+lvpv.selected.subitems[0]);
    ph:=openprocess(PROCESS_TERMINATE,false,pid);
    if (ph<>0) then begin
      if (MessageDlg(lvpv.selected.caption+#13#10#13#10+SWARN,mtWarning,[mbYes,mbNo],0)=mryes) then begin
        terminateprocess(ph,0);
        //initprocesslist;
      end;
    end else
      MessageDlg(SCannotTerm,mtInformation,[mbOK],0);
  end;
end;

procedure TfrmProcView.lvpvKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
var
  n :tlistitem;
begin
  n:=nil;
  if key in [48..126] then
    n:=tlistview(sender).findcaption(0,chr(key),true,true,true);
    if assigned(n) then begin
      n.makevisible(true);
      tlistview(sender).selected:=n;
    end;
end;

procedure TfrmProcView.Refresh;
var
  i,j,c :integer;
  s :string;
begin
  try
    if isnt then
      getntprocesslist(FPerfLib,processlist)
    else
      get95processlist(processlist);
  except
  end;

  with lvpv, Items do begin
    BeginUpdate;
    Clear;
    if IsNT then
      for i:=0 to processlist.count-1 do
        with Add do begin
          caption:=pntprocess(processlist.objects[i])^.Name;
          subitems.add(inttostr(pntprocess(processlist.objects[i])^.PID));
          subitems.add(formatseconds(pntprocess(processlist.objects[i])^.CPUTime,true,false,true));
          subitems.add(formatfloat('0 K',pntprocess(processlist.objects[i])^.WorkingSetSize/1024));
          subitems.add(inttostr(pntprocess(processlist.objects[i])^.PriorityBase));
          subitems.add(inttostr(pntprocess(processlist.objects[i])^.ThreadCount));
        end;
    else
      for i:=0 to processlist.count-1 do
        if processlist[i]<>'' then
          with lvpv.items.add do begin
            caption:=p95process(processlist.objects[i])^.Name;
            subitems.add(inttohex(p95process(processlist.objects[i])^.PID,8));
            subitems.add(getpriority(p95process(processlist.objects[i])^.Priority));
            subitems.add(inttostr(p95process(processlist.objects[i])^.Usage));
            subitems.add(inttostr(p95process(processlist.objects[i])^.ThreadCount));
          end;
    EndUpdate;
  end;
  sb.panels[1].text:=Format('%d',[lvpv.items.count]);
end;

end.
